@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2013 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: unixtime2tm.c 307103 2011-01-05 04:13:23Z iliaa $ */
+/* $Id$ */
 
 #include "timelib.h"
 
@@ -146,7 +146,7 @@ void timelib_update_from_sse(timelib_time *tm)
 			int z = tm->z;
 			signed int dst = tm->dst;
 			
-			timelib_unixtime2gmt(tm, tm->sse - (tm->z * 60));
+			timelib_unixtime2gmt(tm, tm->sse - (tm->z * 60) + (tm->dst * 3600));
 
 			tm->z = z;
 			tm->dst = dst;
@@ -184,7 +184,7 @@ void timelib_unixtime2local(timelib_time *tm, timelib_sll ts)
 			int z = tm->z;
 			signed int dst = tm->dst;
 			
-			timelib_unixtime2gmt(tm, ts - (tm->z * 60));
+			timelib_unixtime2gmt(tm, ts - (tm->z * 60) + (tm->dst * 3600));
 
 			tm->z = z;
 			tm->dst = dst;
