@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2013 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: interbase.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id$ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -43,67 +43,340 @@
 #define COMMIT			1
 #define RETAIN			2
 
+#define CHECK_LINK(link) { if (link==-1) { php_error_docref(NULL TSRMLS_CC, E_WARNING, "A link to the server could not be established"); RETURN_FALSE; } }
+
 ZEND_DECLARE_MODULE_GLOBALS(ibase)
 static PHP_GINIT_FUNCTION(ibase);
 
-/* {{{ extension definition structures */
-zend_function_entry ibase_functions[] = {
-	PHP_FE(ibase_connect, NULL)
-	PHP_FE(ibase_pconnect, NULL)
-	PHP_FE(ibase_close, NULL)
-	PHP_FE(ibase_drop_db, NULL)
-	PHP_FE(ibase_query, NULL)
-	PHP_FE(ibase_fetch_row, NULL)
-	PHP_FE(ibase_fetch_assoc, NULL)
-	PHP_FE(ibase_fetch_object, NULL)
-	PHP_FE(ibase_free_result, NULL)
-	PHP_FE(ibase_name_result, NULL)
-	PHP_FE(ibase_prepare, NULL)
-	PHP_FE(ibase_execute, NULL)
-	PHP_FE(ibase_free_query, NULL)
-	PHP_FE(ibase_gen_id, NULL)
-	PHP_FE(ibase_num_fields, NULL)
-	PHP_FE(ibase_num_params, NULL)
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO(arginfo_ibase_errmsg, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_ibase_errcode, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_connect, 0, 0, 1)
+	ZEND_ARG_INFO(0, database)
+	ZEND_ARG_INFO(0, username)
+	ZEND_ARG_INFO(0, password)
+	ZEND_ARG_INFO(0, charset)
+	ZEND_ARG_INFO(0, buffers)
+	ZEND_ARG_INFO(0, dialect)
+	ZEND_ARG_INFO(0, role)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_pconnect, 0, 0, 1)
+	ZEND_ARG_INFO(0, database)
+	ZEND_ARG_INFO(0, username)
+	ZEND_ARG_INFO(0, password)
+	ZEND_ARG_INFO(0, charset)
+	ZEND_ARG_INFO(0, buffers)
+	ZEND_ARG_INFO(0, dialect)
+	ZEND_ARG_INFO(0, role)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_close, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_drop_db, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_trans, 0, 0, 0)
+	ZEND_ARG_INFO(0, trans_args)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, trans_args)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_commit, 0, 0, 1)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_rollback, 0, 0, 1)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_commit_ret, 0, 0, 1)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_rollback_ret, 0, 0, 1)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_gen_id, 0, 0, 1)
+	ZEND_ARG_INFO(0, generator)
+	ZEND_ARG_INFO(0, increment)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_create, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_open, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, blob_id)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_add, 0, 0, 2)
+	ZEND_ARG_INFO(0, blob_handle)
+	ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_get, 0, 0, 2)
+	ZEND_ARG_INFO(0, blob_handle)
+	ZEND_ARG_INFO(0, len)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_close, 0, 0, 1)
+	ZEND_ARG_INFO(0, blob_handle)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_cancel, 0, 0, 1)
+	ZEND_ARG_INFO(0, blob_handle)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_info, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, blob_id)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_echo, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, blob_id)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_import, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, file)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_query, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, query)
+	ZEND_ARG_INFO(0, bind_arg)
+	ZEND_ARG_INFO(0, bind_arg)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_affected_rows, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
 #if abies_0
-	PHP_FE(ibase_num_rows, NULL)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_num_rows, 0, 0, 1)
+	ZEND_ARG_INFO(0, result_identifier)
+ZEND_END_ARG_INFO()
 #endif
-	PHP_FE(ibase_affected_rows, NULL)
-	PHP_FE(ibase_field_info, NULL)
-	PHP_FE(ibase_param_info, NULL)
 
-	PHP_FE(ibase_trans, NULL)
-	PHP_FE(ibase_commit, NULL)
-	PHP_FE(ibase_rollback, NULL)
-	PHP_FE(ibase_commit_ret, NULL)
-	PHP_FE(ibase_rollback_ret, NULL)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_fetch_row, 0, 0, 1)
+	ZEND_ARG_INFO(0, result)
+	ZEND_ARG_INFO(0, fetch_flags)
+ZEND_END_ARG_INFO()
 
-	PHP_FE(ibase_blob_info, NULL)
-	PHP_FE(ibase_blob_create, NULL)
-	PHP_FE(ibase_blob_add, NULL)
-	PHP_FE(ibase_blob_cancel, NULL)
-	PHP_FE(ibase_blob_close, NULL)
-	PHP_FE(ibase_blob_open, NULL)
-	PHP_FE(ibase_blob_get, NULL)
-	PHP_FE(ibase_blob_echo, NULL)
-	PHP_FE(ibase_blob_import, NULL)
-	PHP_FE(ibase_errmsg, NULL)
-	PHP_FE(ibase_errcode, NULL)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_fetch_assoc, 0, 0, 1)
+	ZEND_ARG_INFO(0, result)
+	ZEND_ARG_INFO(0, fetch_flags)
+ZEND_END_ARG_INFO()
 
-	PHP_FE(ibase_add_user, NULL)
-	PHP_FE(ibase_modify_user, NULL)
-	PHP_FE(ibase_delete_user, NULL)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_fetch_object, 0, 0, 1)
+	ZEND_ARG_INFO(0, result)
+	ZEND_ARG_INFO(0, fetch_flags)
+ZEND_END_ARG_INFO()
 
-	PHP_FE(ibase_service_attach, NULL)
-	PHP_FE(ibase_service_detach, NULL)
-	PHP_FE(ibase_backup, NULL)
-	PHP_FE(ibase_restore, NULL)
-	PHP_FE(ibase_maintain_db, NULL)
-	PHP_FE(ibase_db_info, NULL)
-	PHP_FE(ibase_server_info, NULL)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_name_result, 0, 0, 2)
+	ZEND_ARG_INFO(0, result)
+	ZEND_ARG_INFO(0, name)
+ZEND_END_ARG_INFO()
 
-	PHP_FE(ibase_wait_event, NULL)
-	PHP_FE(ibase_set_event_handler, NULL)
-	PHP_FE(ibase_free_event_handler, NULL)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_free_result, 0, 0, 1)
+	ZEND_ARG_INFO(0, result)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_prepare, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, query)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_execute, 0, 0, 1)
+	ZEND_ARG_INFO(0, query)
+	ZEND_ARG_INFO(0, bind_arg)
+	ZEND_ARG_INFO(0, bind_arg)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_free_query, 0, 0, 1)
+	ZEND_ARG_INFO(0, query)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_num_fields, 0, 0, 1)
+	ZEND_ARG_INFO(0, query_result)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_field_info, 0, 0, 2)
+	ZEND_ARG_INFO(0, query_result)
+	ZEND_ARG_INFO(0, field_number)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_num_params, 0, 0, 1)
+	ZEND_ARG_INFO(0, query)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_param_info, 0, 0, 2)
+	ZEND_ARG_INFO(0, query)
+	ZEND_ARG_INFO(0, field_number)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_add_user, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, user_name)
+	ZEND_ARG_INFO(0, password)
+	ZEND_ARG_INFO(0, first_name)
+	ZEND_ARG_INFO(0, middle_name)
+	ZEND_ARG_INFO(0, last_name)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_modify_user, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, user_name)
+	ZEND_ARG_INFO(0, password)
+	ZEND_ARG_INFO(0, first_name)
+	ZEND_ARG_INFO(0, middle_name)
+	ZEND_ARG_INFO(0, last_name)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_delete_user, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, user_name)
+	ZEND_ARG_INFO(0, password)
+	ZEND_ARG_INFO(0, first_name)
+	ZEND_ARG_INFO(0, middle_name)
+	ZEND_ARG_INFO(0, last_name)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_service_attach, 0, 0, 3)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, dba_username)
+	ZEND_ARG_INFO(0, dba_password)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_service_detach, 0, 0, 1)
+	ZEND_ARG_INFO(0, service_handle)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_backup, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, source_db)
+	ZEND_ARG_INFO(0, dest_file)
+	ZEND_ARG_INFO(0, options)
+	ZEND_ARG_INFO(0, verbose)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_restore, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, source_file)
+	ZEND_ARG_INFO(0, dest_db)
+	ZEND_ARG_INFO(0, options)
+	ZEND_ARG_INFO(0, verbose)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_maintain_db, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, db)
+	ZEND_ARG_INFO(0, action)
+	ZEND_ARG_INFO(0, argument)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_db_info, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, db)
+	ZEND_ARG_INFO(0, action)
+	ZEND_ARG_INFO(0, argument)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_server_info, 0, 0, 2)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, action)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_wait_event, 0, 0, 1)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, event)
+	ZEND_ARG_INFO(0, event2)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_set_event_handler, 0, 0, 2)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, handler)
+	ZEND_ARG_INFO(0, event)
+	ZEND_ARG_INFO(0, event2)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_free_event_handler, 0, 0, 1)
+	ZEND_ARG_INFO(0, event)
+ZEND_END_ARG_INFO()
+/* }}} */
+
+/* {{{ extension definition structures */
+const zend_function_entry ibase_functions[] = {
+	PHP_FE(ibase_connect, 		arginfo_ibase_connect)
+	PHP_FE(ibase_pconnect, 		arginfo_ibase_pconnect)
+	PHP_FE(ibase_close, 		arginfo_ibase_close)
+	PHP_FE(ibase_drop_db, 		arginfo_ibase_drop_db)
+	PHP_FE(ibase_query, 		arginfo_ibase_query)
+	PHP_FE(ibase_fetch_row, 	arginfo_ibase_fetch_row)
+	PHP_FE(ibase_fetch_assoc, 	arginfo_ibase_fetch_assoc)
+	PHP_FE(ibase_fetch_object, 	arginfo_ibase_fetch_object)
+	PHP_FE(ibase_free_result, 	arginfo_ibase_free_result)
+	PHP_FE(ibase_name_result, 	arginfo_ibase_name_result)
+	PHP_FE(ibase_prepare, 		arginfo_ibase_prepare)
+	PHP_FE(ibase_execute, 		arginfo_ibase_execute)
+	PHP_FE(ibase_free_query, 	arginfo_ibase_free_query)
+	PHP_FE(ibase_gen_id, 		arginfo_ibase_gen_id)
+	PHP_FE(ibase_num_fields, 	arginfo_ibase_num_fields)
+	PHP_FE(ibase_num_params, 	arginfo_ibase_num_params)
+#if abies_0
+	PHP_FE(ibase_num_rows, 		arginfo_ibase_num_rows)
+#endif
+	PHP_FE(ibase_affected_rows, arginfo_ibase_affected_rows)
+	PHP_FE(ibase_field_info, 	arginfo_ibase_field_info)
+	PHP_FE(ibase_param_info, 	arginfo_ibase_param_info)
+
+	PHP_FE(ibase_trans, 		arginfo_ibase_trans)
+	PHP_FE(ibase_commit, 		arginfo_ibase_commit)
+	PHP_FE(ibase_rollback, 		arginfo_ibase_rollback)
+	PHP_FE(ibase_commit_ret, 	arginfo_ibase_commit_ret)
+	PHP_FE(ibase_rollback_ret, 	arginfo_ibase_rollback_ret)
+
+	PHP_FE(ibase_blob_info, 	arginfo_ibase_blob_info)
+	PHP_FE(ibase_blob_create, 	arginfo_ibase_blob_create)
+	PHP_FE(ibase_blob_add, 		arginfo_ibase_blob_add)
+	PHP_FE(ibase_blob_cancel, 	arginfo_ibase_blob_cancel)
+	PHP_FE(ibase_blob_close, 	arginfo_ibase_blob_close)
+	PHP_FE(ibase_blob_open, 	arginfo_ibase_blob_open)
+	PHP_FE(ibase_blob_get, 		arginfo_ibase_blob_get)
+	PHP_FE(ibase_blob_echo, 	arginfo_ibase_blob_echo)
+	PHP_FE(ibase_blob_import, 	arginfo_ibase_blob_import)
+	PHP_FE(ibase_errmsg, 		arginfo_ibase_errmsg)
+	PHP_FE(ibase_errcode, 		arginfo_ibase_errcode)
+
+	PHP_FE(ibase_add_user, 		arginfo_ibase_add_user)
+	PHP_FE(ibase_modify_user, 	arginfo_ibase_modify_user)
+	PHP_FE(ibase_delete_user, 	arginfo_ibase_delete_user)
+
+	PHP_FE(ibase_service_attach, arginfo_ibase_service_attach)
+	PHP_FE(ibase_service_detach, arginfo_ibase_service_detach)
+	PHP_FE(ibase_backup, 		arginfo_ibase_backup)
+	PHP_FE(ibase_restore, 		arginfo_ibase_restore)
+	PHP_FE(ibase_maintain_db, 	arginfo_ibase_maintain_db)
+	PHP_FE(ibase_db_info, 		arginfo_ibase_db_info)
+	PHP_FE(ibase_server_info, 	arginfo_ibase_server_info)
+
+	PHP_FE(ibase_wait_event, 			arginfo_ibase_wait_event)
+	PHP_FE(ibase_set_event_handler, 	arginfo_ibase_set_event_handler)
+	PHP_FE(ibase_free_event_handler, 	arginfo_ibase_free_event_handler)
 
 	/**
 	* These aliases are provided in order to maintain forward compatibility. As Firebird
@@ -112,63 +385,63 @@ zend_function_entry ibase_functions[] = {
 	* Firebird users should use the aliases, so future InterBase-specific changes will
 	* not affect their code
 	*/
-	PHP_FALIAS(fbird_connect,ibase_connect, NULL)
-	PHP_FALIAS(fbird_pconnect,ibase_pconnect, NULL)
-	PHP_FALIAS(fbird_close,ibase_close, NULL)
-	PHP_FALIAS(fbird_drop_db,ibase_drop_db, NULL)
-	PHP_FALIAS(fbird_query,ibase_query, NULL)
-	PHP_FALIAS(fbird_fetch_row,ibase_fetch_row, NULL)
-	PHP_FALIAS(fbird_fetch_assoc,ibase_fetch_assoc, NULL)
-	PHP_FALIAS(fbird_fetch_object,ibase_fetch_object, NULL)
-	PHP_FALIAS(fbird_free_result,ibase_free_result, NULL)
-	PHP_FALIAS(fbird_name_result,ibase_name_result, NULL)
-	PHP_FALIAS(fbird_prepare,ibase_prepare, NULL)
-	PHP_FALIAS(fbird_execute,ibase_execute, NULL)
-	PHP_FALIAS(fbird_free_query,ibase_free_query, NULL)
-	PHP_FALIAS(fbird_gen_id,ibase_gen_id, NULL)
-	PHP_FALIAS(fbird_num_fields,ibase_num_fields, NULL)
-	PHP_FALIAS(fbird_num_params,ibase_num_params, NULL)
+	PHP_FALIAS(fbird_connect,		ibase_connect, 		arginfo_ibase_connect)
+	PHP_FALIAS(fbird_pconnect,		ibase_pconnect, 	arginfo_ibase_pconnect)
+	PHP_FALIAS(fbird_close,			ibase_close, 		arginfo_ibase_close)
+	PHP_FALIAS(fbird_drop_db,		ibase_drop_db, 		arginfo_ibase_drop_db)
+	PHP_FALIAS(fbird_query,			ibase_query, 		arginfo_ibase_query)
+	PHP_FALIAS(fbird_fetch_row,		ibase_fetch_row, 	arginfo_ibase_fetch_row)
+	PHP_FALIAS(fbird_fetch_assoc,	ibase_fetch_assoc, 	arginfo_ibase_fetch_assoc)
+	PHP_FALIAS(fbird_fetch_object,	ibase_fetch_object, arginfo_ibase_fetch_object)
+	PHP_FALIAS(fbird_free_result,	ibase_free_result, 	arginfo_ibase_free_result)
+	PHP_FALIAS(fbird_name_result,	ibase_name_result, 	arginfo_ibase_name_result)
+	PHP_FALIAS(fbird_prepare,		ibase_prepare, 		arginfo_ibase_prepare)
+	PHP_FALIAS(fbird_execute,		ibase_execute, 		arginfo_ibase_execute)
+	PHP_FALIAS(fbird_free_query,	ibase_free_query, 	arginfo_ibase_free_query)
+	PHP_FALIAS(fbird_gen_id,		ibase_gen_id, 		arginfo_ibase_gen_id)
+	PHP_FALIAS(fbird_num_fields,	ibase_num_fields, 	arginfo_ibase_num_fields)
+	PHP_FALIAS(fbird_num_params,	ibase_num_params, 	arginfo_ibase_num_params)
 #if abies_0
-	PHP_FALIAS(fbird_num_rows,ibase_num_rows, NULL)
+	PHP_FALIAS(fbird_num_rows,		ibase_num_rows, 	arginfo_ibase_num_rows)
 #endif
-	PHP_FALIAS(fbird_affected_rows,ibase_affected_rows, NULL)
-	PHP_FALIAS(fbird_field_info,ibase_field_info, NULL)
-	PHP_FALIAS(fbird_param_info,ibase_param_info, NULL)
+	PHP_FALIAS(fbird_affected_rows,	ibase_affected_rows, arginfo_ibase_affected_rows)
+	PHP_FALIAS(fbird_field_info,	ibase_field_info, 	arginfo_ibase_field_info)
+	PHP_FALIAS(fbird_param_info,	ibase_param_info, 	arginfo_ibase_param_info)
 
-	PHP_FALIAS(fbird_trans,ibase_trans, NULL)
-	PHP_FALIAS(fbird_commit,ibase_commit, NULL)
-	PHP_FALIAS(fbird_rollback,ibase_rollback, NULL)
-	PHP_FALIAS(fbird_commit_ret,ibase_commit_ret, NULL)
-	PHP_FALIAS(fbird_rollback_ret,ibase_rollback_ret, NULL)
+	PHP_FALIAS(fbird_trans,			ibase_trans, 		arginfo_ibase_trans)
+	PHP_FALIAS(fbird_commit,		ibase_commit, 		arginfo_ibase_commit)
+	PHP_FALIAS(fbird_rollback,		ibase_rollback, 	arginfo_ibase_rollback)
+	PHP_FALIAS(fbird_commit_ret,	ibase_commit_ret, 	arginfo_ibase_commit_ret)
+	PHP_FALIAS(fbird_rollback_ret,	ibase_rollback_ret, arginfo_ibase_rollback_ret)
 
-	PHP_FALIAS(fbird_blob_info,ibase_blob_info, NULL)
-	PHP_FALIAS(fbird_blob_create,ibase_blob_create, NULL)
-	PHP_FALIAS(fbird_blob_add,ibase_blob_add, NULL)
-	PHP_FALIAS(fbird_blob_cancel,ibase_blob_cancel, NULL)
-	PHP_FALIAS(fbird_blob_close,ibase_blob_close, NULL)
-	PHP_FALIAS(fbird_blob_open,ibase_blob_open, NULL)
-	PHP_FALIAS(fbird_blob_get,ibase_blob_get, NULL)
-	PHP_FALIAS(fbird_blob_echo,ibase_blob_echo, NULL)
-	PHP_FALIAS(fbird_blob_import,ibase_blob_import, NULL)
-	PHP_FALIAS(fbird_errmsg,ibase_errmsg, NULL)
-	PHP_FALIAS(fbird_errcode,ibase_errcode, NULL)
+	PHP_FALIAS(fbird_blob_info,		ibase_blob_info, 	arginfo_ibase_blob_info)
+	PHP_FALIAS(fbird_blob_create,	ibase_blob_create, 	arginfo_ibase_blob_create)
+	PHP_FALIAS(fbird_blob_add,		ibase_blob_add, 	arginfo_ibase_blob_add)
+	PHP_FALIAS(fbird_blob_cancel,	ibase_blob_cancel, 	arginfo_ibase_blob_cancel)
+	PHP_FALIAS(fbird_blob_close,	ibase_blob_close, 	arginfo_ibase_blob_close)
+	PHP_FALIAS(fbird_blob_open,		ibase_blob_open, 	arginfo_ibase_blob_open)
+	PHP_FALIAS(fbird_blob_get,		ibase_blob_get, 	arginfo_ibase_blob_get)
+	PHP_FALIAS(fbird_blob_echo,		ibase_blob_echo, 	arginfo_ibase_blob_echo)
+	PHP_FALIAS(fbird_blob_import,	ibase_blob_import, 	arginfo_ibase_blob_import)
+	PHP_FALIAS(fbird_errmsg,		ibase_errmsg, 		arginfo_ibase_errmsg)
+	PHP_FALIAS(fbird_errcode,		ibase_errcode, 		arginfo_ibase_errcode)
 
-	PHP_FALIAS(fbird_add_user,ibase_add_user, NULL)
-	PHP_FALIAS(fbird_modify_user,ibase_modify_user, NULL)
-	PHP_FALIAS(fbird_delete_user,ibase_delete_user, NULL)
+	PHP_FALIAS(fbird_add_user,		ibase_add_user, 	arginfo_ibase_add_user)
+	PHP_FALIAS(fbird_modify_user,	ibase_modify_user, 	arginfo_ibase_modify_user)
+	PHP_FALIAS(fbird_delete_user,	ibase_delete_user, 	arginfo_ibase_delete_user)
 
-	PHP_FALIAS(fbird_service_attach,ibase_service_attach, NULL)
-	PHP_FALIAS(fbird_service_detach,ibase_service_detach, NULL)
-	PHP_FALIAS(fbird_backup,ibase_backup, NULL)
-	PHP_FALIAS(fbird_restore,ibase_restore, NULL)
-	PHP_FALIAS(fbird_maintain_db,ibase_maintain_db, NULL)
-	PHP_FALIAS(fbird_db_info,ibase_db_info, NULL)
-	PHP_FALIAS(fbird_server_info,ibase_server_info, NULL)
+	PHP_FALIAS(fbird_service_attach,	ibase_service_attach, arginfo_ibase_service_attach)
+	PHP_FALIAS(fbird_service_detach,	ibase_service_detach, arginfo_ibase_service_detach)
+	PHP_FALIAS(fbird_backup,		ibase_backup, 		arginfo_ibase_backup)
+	PHP_FALIAS(fbird_restore,		ibase_restore, 		arginfo_ibase_restore)
+	PHP_FALIAS(fbird_maintain_db,	ibase_maintain_db, 	arginfo_ibase_maintain_db)
+	PHP_FALIAS(fbird_db_info,		ibase_db_info, 		arginfo_ibase_db_info)
+	PHP_FALIAS(fbird_server_info,	ibase_server_info, 	arginfo_ibase_server_info)
 
-	PHP_FALIAS(fbird_wait_event,ibase_wait_event, NULL)
-	PHP_FALIAS(fbird_set_event_handler,ibase_set_event_handler, NULL)
-	PHP_FALIAS(fbird_free_event_handler,ibase_free_event_handler, NULL)
-	{NULL, NULL, NULL}
+	PHP_FALIAS(fbird_wait_event,	ibase_wait_event, 	arginfo_ibase_wait_event)
+	PHP_FALIAS(fbird_set_event_handler,	ibase_set_event_handler, 	arginfo_ibase_set_event_handler)
+	PHP_FALIAS(fbird_free_event_handler,	ibase_free_event_handler, arginfo_ibase_free_event_handler)
+	PHP_FE_END
 };
 
 zend_module_entry ibase_module_entry = {
@@ -203,8 +476,8 @@ int le_link, le_plink, le_trans;
    Return error message */
 PHP_FUNCTION(ibase_errmsg)
 {
-	if (ZEND_NUM_ARGS() != 0) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 
 	if (IBG(sql_code) != 0) {
@@ -219,8 +492,8 @@ PHP_FUNCTION(ibase_errmsg)
    Return error code */
 PHP_FUNCTION(ibase_errcode)
 {
-	if (ZEND_NUM_ARGS() != 0) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 
 	if (IBG(sql_code) != 0) {
@@ -725,9 +998,12 @@ static void _php_ibase_connect(INTERNAL_FUNCTION_PARAMETERS, int persistent) /*
 			ZEND_REGISTER_RESOURCE(return_value, ib_link, le_link);
 		} else {
 			zend_rsrc_list_entry new_le;
-			
+
 			ib_link = (ibase_db_link *) malloc(sizeof(ibase_db_link));
-	
+			if (!ib_link) {
+				RETURN_FALSE;
+			}
+
 			/* hash it up */
 			Z_TYPE(new_le) = le_plink;
 			new_le.ptr = ib_link;
@@ -778,30 +1054,25 @@ PHP_FUNCTION(ibase_pconnect)
    Close an InterBase connection */
 PHP_FUNCTION(ibase_close)
 {
-	zval **link_arg = NULL;
+	zval *link_arg = NULL;
 	ibase_db_link *ib_link;
 	int link_id;
 
 	RESET_ERRMSG;
 	
-	switch (ZEND_NUM_ARGS()) {
-		case 0:
-			link_id = IBG(default_link);
-			IBG(default_link) = -1;
-			break;
-		case 1:
-			if (zend_get_parameters_ex(1, &link_arg) == FAILURE) {
-				RETURN_FALSE;
-			}
-			convert_to_long_ex(link_arg);
-			link_id = Z_LVAL_PP(link_arg);
-			break;
-		default:
-			WRONG_PARAM_COUNT;
-			break;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|r", &link_arg) == FAILURE) {
+		return;
+	}
+	
+	if (ZEND_NUM_ARGS() == 0) {
+		link_id = IBG(default_link);
+		CHECK_LINK(link_id);
+		IBG(default_link) = -1;
+	} else {
+		link_id = Z_RESVAL_P(link_arg);
 	}
 
-	ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, link_arg, link_id, LE_LINK, le_link, le_plink);
+	ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, &link_arg, link_id, LE_LINK, le_link, le_plink);
 	zend_list_delete(link_id);
 	RETURN_TRUE;
 }
@@ -811,31 +1082,26 @@ PHP_FUNCTION(ibase_close)
    Drop an InterBase database */
 PHP_FUNCTION(ibase_drop_db)
 {
-	zval **link_arg = NULL;
+	zval *link_arg = NULL;
 	ibase_db_link *ib_link;
 	ibase_tr_list *l;
 	int link_id;
 
 	RESET_ERRMSG;
-
-	switch (ZEND_NUM_ARGS()) {
-		case 0:
-			link_id = IBG(default_link);
-			IBG(default_link) = -1;
-			break;
-		case 1:
-			if (zend_get_parameters_ex(1, &link_arg) == FAILURE) {
-				RETURN_FALSE;
-			}
-			convert_to_long_ex(link_arg);
-			link_id = Z_LVAL_PP(link_arg);
-			break;
-		default:
-			WRONG_PARAM_COUNT;
-			break;
+	
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|r", &link_arg) == FAILURE) {
+		return;
 	}
 	
-	ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, link_arg, link_id, LE_LINK, le_link, le_plink);
+	if (ZEND_NUM_ARGS() == 0) {
+		link_id = IBG(default_link);
+		CHECK_LINK(link_id);
+		IBG(default_link) = -1;
+	} else {
+		link_id = Z_RESVAL_P(link_arg);
+	}
+	
+	ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, &link_arg, link_id, LE_LINK, le_link, le_plink);
 
 	if (isc_drop_database(IB_STATUS, &ib_link->handle)) {
 		_php_ibase_error(TSRMLS_C);
@@ -859,7 +1125,8 @@ PHP_FUNCTION(ibase_drop_db)
 
 PHP_FUNCTION(ibase_trans)
 {
-	unsigned short i, argn, link_cnt = 0, tpb_len = 0;
+	unsigned short i, link_cnt = 0, tpb_len = 0;
+	int argn;
 	char last_tpb[TPB_MAX_SIZE];
 	ibase_db_link **ib_link = NULL;
 	ibase_trans *ib_trans;
@@ -877,9 +1144,9 @@ PHP_FUNCTION(ibase_trans)
 		long trans_argl = 0;
 		char *tpb;
 		ISC_TEB *teb;
-		zval ***args = (zval ***) safe_emalloc(sizeof(zval **),argn,0);
+		zval ***args = NULL;
 
-		if (zend_get_parameters_array_ex(argn, args) == FAILURE) {
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "+", &args, &argn) == FAILURE) {
 			efree(args);
 			efree(ib_link);
 			RETURN_FALSE;
@@ -1047,55 +1314,42 @@ static void _php_ibase_trans_end(INTERNAL_FUNCTION_PARAMETERS, int commit) /* {{
 	ibase_trans *trans = NULL;
 	int res_id = 0;
 	ISC_STATUS result;
+	ibase_db_link *ib_link;
+	zval *arg = NULL;
+	int type;
 
 	RESET_ERRMSG;
+	
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|r", &arg) == FAILURE) {
+		return;
+	}
 
-	switch (ZEND_NUM_ARGS()) {
+	if (ZEND_NUM_ARGS() == 0) {
+		ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, NULL, IBG(default_link), LE_LINK, le_link, le_plink);
+		if (ib_link->tr_list == NULL || ib_link->tr_list->trans == NULL) {
+			/* this link doesn't have a default transaction */
+			_php_ibase_module_error("Default link has no default transaction" TSRMLS_CC);
+			RETURN_FALSE;
+		}
+		trans = ib_link->tr_list->trans;
+	} else {
+		/* one id was passed, could be db or trans id */
+		if (zend_list_find(Z_RESVAL_P(arg), &type) && type == le_trans) {
+			ZEND_FETCH_RESOURCE(trans, ibase_trans *, &arg, -1, LE_TRANS, le_trans);
+			res_id = Z_RESVAL_P(arg);
+		} else {
+			ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, &arg, -1, LE_LINK, le_link, le_plink);
 
-		ibase_db_link *ib_link;
-		zval **arg;
-		int type;
-
-		case 0:
-			ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, NULL, IBG(default_link), LE_LINK, 
-				le_link, le_plink);
 			if (ib_link->tr_list == NULL || ib_link->tr_list->trans == NULL) {
 				/* this link doesn't have a default transaction */
-				_php_ibase_module_error("Default link has no default transaction" TSRMLS_CC);
+				_php_ibase_module_error("Link has no default transaction" TSRMLS_CC);
 				RETURN_FALSE;
 			}
 			trans = ib_link->tr_list->trans;
-			break;
-
-		case 1: 
-			if (zend_get_parameters_ex(1, &arg) == FAILURE) {
-				RETURN_FALSE;
-			}
-			/* one id was passed, could be db or trans id */
-			if (zend_list_find(Z_LVAL_PP(arg), &type) && type == le_trans) {			
-				ZEND_FETCH_RESOURCE(trans, ibase_trans *, arg, -1, LE_TRANS, le_trans);
-
-				convert_to_long_ex(arg);
-				res_id = Z_LVAL_PP(arg);
-			} else {
-				ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, arg, -1, LE_LINK, le_link, le_plink);
-
-				if (ib_link->tr_list == NULL || ib_link->tr_list->trans == NULL) {
-					/* this link doesn't have a default transaction */
-					_php_ibase_module_error("Link has no default transaction" TSRMLS_CC);
-					RETURN_FALSE;
-				}
-				trans = ib_link->tr_list->trans;
-			}
-			break;
-
-		default:
-			WRONG_PARAM_COUNT;
-			break;
+		}
 	}
 
 	switch (commit) {
-		
 		default: /* == case ROLLBACK: */
 			result = isc_rollback_transaction(IB_STATUS, &trans->handle);
 			break;
@@ -1116,7 +1370,7 @@ static void _php_ibase_trans_end(INTERNAL_FUNCTION_PARAMETERS, int commit) /* {{
 	}
 
 	/* Don't try to destroy implicitly opened transaction from list... */
-	if ( (commit & RETAIN) == 0 && res_id != 0) {
+	if ((commit & RETAIN) == 0 && res_id != 0) {
 		zend_list_delete(res_id);
 	}
 	RETURN_TRUE;
@@ -1174,6 +1428,11 @@ PHP_FUNCTION(ibase_gen_id)
 			&inc, &link)) {
 		RETURN_FALSE;
 	}
+	
+	if (gen_len > 31) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid generator name");
+		RETURN_FALSE;
+	}
 
 	PHP_IBASE_LINK_TRANS(link, ib_link, trans);
 	
