@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2011 The PHP Group                                |
+   | Copyright (c) 1997-2013 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: pspell.c 306939 2011-01-01 02:19:59Z felipe $ */
+/* $Id$ */
 
 #define IS_EXT_MODULE
 
@@ -198,7 +198,7 @@ static const zend_function_entry pspell_functions[] = {
 	PHP_FE(pspell_config_data_dir,		arginfo_pspell_config_data_dir)
 	PHP_FE(pspell_config_repl,			arginfo_pspell_config_repl)
 	PHP_FE(pspell_config_save_repl,		arginfo_pspell_config_save_repl)
-	{NULL, NULL, NULL} 
+	PHP_FE_END
 };
 /* }}} */
 
